# Group remove command.

desc = 'Removes a sound from a group.'
usage = 'groupdel <GROUPNAME> <CODE|ALIAS>'

def execute(data, argv):
    c = data.db.conn.cursor()

    if len(argv) < 2:
        raise Exception('expected [group] [sound]')

    if ':' in argv[1]:
        raise Exception('no codes with \':\' allowed')

    c.execute('SELECT content FROM groups WHERE groupname=?', [argv[0]])

    res = c.fetchall()

    if len(res) == 0:
        data.reply('Group {} not found'.format(argv[0]))
    else:
        old_content = res[0][0].split(':')

        if argv[1] in old_content:
            old_content.remove(argv[1])
        else:
            data.reply('WARNING: {} not in group'.format(argv[1]))

        new_content = ':'.join(old_content)

        if len(new_content) > 0:
            c.execute('UPDATE groups SET content=? WHERE groupname=?', [new_content, argv[0]])
            data.reply('Removed {} from group {}.'.format(argv[1], argv[2]))
        else:
            c.execute('DELETE FROM groups WHERE groupname=?', [argv[0]])

        data.db.conn.commit()


# Group play command.

import random

desc = 'Plays a random sound from a group.'
usage = 'gplay <GROUPNAME>'

def execute(data, argv):
    if len(argv) < 1:
        raise Exception('expected argument')

    # check group exists

    # select a random sound
    c = data.db.conn.cursor()
    c.execute('SELECT content FROM groups WHERE groupname=?', [argv[0]])
    
    res = c.fetchall()
    
    if len(res) == 0:
        raise Exception('"{}": group not found'.format(argv[0]))

    to_play = random.choice(res[0][0].split(':'))
    data.util.play_sound_or_alias(to_play)
    data.reply('Playing {}.'.format(to_play))


# Sound query command.

desc = 'Lists available sounds.'
usage = 'sounds [PAGENUM]'

def execute(data, argv):
    # query all sounds
    c = data.db.conn.cursor()
    c.execute('SELECT * FROM sounds ORDER BY timestamp DESC')
    rows = c.fetchall()

    pages = data.util.into_pages(['Sound', 'Author', 'Created'], rows)
    selected = int(argv[0]) - 1 if len(argv) > 0 else 0

    if selected < 0 or selected >= len(pages):
        raise Exception('invalid page number')

    data.reply('Showing page {} of {}'.format(selected + 1, len(pages)))
    data.reply(pages[selected])


# Sound play command.

desc = "Plays a sound from the local collection."
usage = "s [CODE]"

def execute(data, argv):
    target = None

    if len(argv) == 0:
        # pick a random sound
        c = data.db.conn.cursor()
        c.execute('SELECT * FROM sounds ORDER BY random() LIMIT 1')
        target = c.fetchone()[0]
    else:
        target = argv[0]
    
    data.audio.play(target)
    data.reply('Playing {}.'.format(target))



# Link query command.

desc = 'Retrieves a random link.'

def execute(data, argv):
    # choose a random link
    c = data.db.conn.cursor()
    c.execute('SELECT * FROM links ORDER BY random() LIMIT 1')

    row = c.fetchone()

    if row is None:
        data.reply('No links!')
    else:
        data.bcast('A gift from <a href="{0}">{1}</a>'.format(row[1], row[0]))


# Greeting command.

desc = 'Sets or unsets your greeting sound.'
usage = 'greeting [CODE|ALIAS]'

def execute(data, argv):
    c = data.db.conn.cursor()

    if len(argv) > 0:
        # check if username is already in db
        c.execute('SELECT * FROM greetings WHERE username=?', [data.author])

        if len(c.fetchall()) == 0:
            c.execute('INSERT INTO greetings VALUES (?, ?)', [data.author, argv[0]])
        else:
            c.execute('UPDATE greetings SET greeting=? WHERE username=?', [argv[0], data.author])

        data.db.conn.commit()
        data.reply('Set greeting to {0}.'.format(argv[0]))
    else:
        c.execute('DELETE FROM greetings WHERE username=?', [data.author])
        data.reply('Removed greeting.')


# Alias command.

desc = 'Defines a new alias.'
usage = 'alias <ALIASNAME> <COMMAND>'

def execute(data, argv):
    if len(argv) < 2:
        raise Exception('expected 2 arguments, found {0}'.format(len(argv) - 1))

    commandname = argv[0]
    action = ' '.join(argv[1:])

    # check if alias already exists
    if data.db.resolve_alias(commandname) is not None:
        raise Exception('destination alias "{0}" already exists!'.format(commandname))

    # create new alias
    c = data.db.conn.cursor()
    c.execute('INSERT INTO aliases VALUES (?, ?, ?, datetime("NOW"))', (commandname, action, data.author))
    data.db.conn.commit()

    data.reply('Created alias "{0}" => "{1}".'.format(commandname, action))


# Alias delete command.

desc = 'Deletes an alias.'
usage = 'delalias <ALIASNAME>'

def execute(data, argv):
    # delete an alias

    if len(argv) < 1:
        raise Exception('expected argument')

    res = data.db.resolve_alias(argv[0])

    if res is None:
        raise Exception('"{0}": alias not found'.format(argv[0]))
    else:
        c = data.db.conn.cursor()
        c.execute('DELETE FROM aliases WHERE commandname=?', [argv[0]])
        data.db.conn.commit()

        data.reply('Deleted alias "{0}".'.format(argv[0]))


# Sound delete command.

import os

desc = 'Deletes a sound clip.'
usage = 'delsound <CODE>'

def execute(data, argv):
    # delete a sound

    if len(argv) < 1:
        raise Exception('expected argument')

    if os.path.exists('sounds/{0}.mp3'.format(argv[0])):
        c = data.db.conn.cursor()
        c.execute('DELETE FROM sounds WHERE soundname=?', [argv[0]])
        data.db.conn.commit()

        os.unlink('sounds/{0}.mp3'.format(argv[0]))
        data.reply('Deleted sound {0}.'.format(argv[0]))
    else:
        raise Exception('"{0}": sound not found'.format(argv[0]))


# Group add command.

desc = 'Adds a sound to a group.'
usage = 'groupadd <GROUPNAME> <CODE|ALIAS>'

def execute(data, argv):
    c = data.db.conn.cursor()

    if len(argv) < 2:
        raise Exception('expected [group] [sound]')

    if ':' in argv[1]:
        raise Exception('no codes with \':\' allowed')

    c.execute('SELECT content FROM groups WHERE groupname=?', [argv[0]])

    res = c.fetchall()

    if len(res) == 0:
        c.execute('INSERT INTO groups VALUES (?, ?, ?, datetime("NOW"))', [argv[0], argv[1], data.author])
        data.db.conn.commit()
        data.reply('Created new group {}'.format(argv[0]))
    else:
        new_content = ':'.join(res[0][0].split(':') + [argv[1]])
        c.execute('UPDATE groups SET content=? WHERE groupname=?', [new_content, argv[0]])
        data.db.conn.commit()
        data.reply('Added {} to group {}.'.format(argv[1], argv[0]))


# Walk through command sources in this directory and initialize them.

import importlib
import os
from os import path

MAX_DEPTH=32

dir_path = os.path.dirname(os.path.realpath(__file__))
command_dict = {}

print('Registering commands..')

for f in os.scandir(dir_path):
    if path.isfile(f.path) and path.basename(f.path) != '__init__.py':
        name = str(path.basename(f.path)).split('.')[0]
        spec = importlib.util.spec_from_file_location(name, str(f.path))
        module = importlib.util.module_from_spec(spec)
        spec.loader.exec_module(module)

        if not hasattr(module, 'execute'):
            raise RuntimeError('Command {} does not have an execute() method!'.format(name))

        if not callable(module.execute):
            raise RuntimeError('Command {} "execute" is not callable!'.format(name))

        print('    > {}'.format(name))
        command_dict[name] = module

print('Registered {} command{}.'.format(
    len(command_dict),
    's' if len(command_dict) > 1 else ''
))

# Register built-in help command

def execute_help(data, argv):
    rows = []

    def esc(s: str):
        s = s.replace('<', '&lt;')
        s = s.replace('>', '&gt;')
        return s

    for name in sorted(command_dict.keys()):
        if len(argv) > 0 and name not in argv:
            continue

        rows.append([
            name,
            command_dict[name].desc,
            esc(command_dict[name].usage) if hasattr(command_dict[name], 'usage') else '',
        ])

    pages = data.util.into_pages(['Command', 'Description', 'Usage'], rows, 32)

    for p in pages:
        data.reply(p)

command_dict['help'] = lambda: None
command_dict['help'].desc = 'Gets help information on one or more commands.'
command_dict['help'].execute = execute_help

def execute(data, argv, depth=0):
    if depth > MAX_DEPTH:
        raise Exception('maximum command depth exceeded')

    # Try and resolve a built-in command
    if argv[0] in command_dict:
        try:
            return command_dict[argv[0]].execute(data, argv[1:])
        except Exception as e:
            raise Exception('{}: {}'.format(argv[0], e))
    
    # Try and resolve an alias
    alias = data.db.resolve_alias(argv[0])

    if alias is None:
        raise Exception('{} is not a recognized command or alias'.format(argv[0]))

    next_argv = alias[1].split(' ') + argv[1:]

    # Drop command indicators from expanded argv
    while next_argv[0][0] == '!':
        next_argv[0] = next_argv[0][1:]

    # Expand alias and recombine arguments
    return execute(data, next_argv, depth + 1)



import setuptools

with open("README.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()

setuptools.setup(
    name="threebot-codeandkey",
    version="0.2.1",
    author="Justin Stanley",
    author_email="jtst@iastate.edu",
    description="A modular mumble bot",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/codeandkey/threebot",
    project_urls={
        "Bug Tracker": "https://github.com/codeandkey/threebot/issues",
    },
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    package_dir={"": "threebot"},
    packages=setuptools.find_packages(where="threebot"),
    python_requires=">=3.6",
    install_requires=[
        'pyaudio',
        'pymumble'
    ],
    scripts=['threebot.py'],
)



# Sound grab command.

import os
import random
import subprocess as sp

desc = 'Grabs a sound clip from YouTube or other internet media source.'
usage = 'get <URL> <[[HR:]MN:]SEC> <DURATION>'

def execute(data, argv):
    if len(argv) < 3:
        raise Exception('get: expected 3 arguments, found {0}'.format(len(argv)))

    # define name generator
    def namegen():
        output = ''

        for i in range(4):
            output += chr(65 + random.randint(0, 25))

        return output

    # find a new ID
    name = namegen()

    while os.path.exists('sounds/{0}.mp3'.format(name)):
        name = namegen()

    # clip a youtube video
    command = ['youtube-dl', '-x',
                '--audio-format', 'mp3',
                '-o', 'sounds/{0}.tmp.mp3'.format(name), argv[0]]
    ytdl = sp.check_output(command, stderr=sp.PIPE)

    # convert clip to desired bounds
    command = ['ffmpeg', '-i', 'sounds/{0}.tmp.mp3'.format(name),
                '-ss', argv[1], '-t', argv[2], 'sounds/{0}.mp3'.format(name)]

    fmpg = sp.check_output(command, stderr=sp.PIPE)

    # remove temporary file
    os.unlink('sounds/{0}.tmp.mp3'.format(name))

    # clip OK, add to database
    c = data.db.conn.cursor()
    c.execute('INSERT INTO sounds VALUES (?, ?, datetime("NOW"))', (name, data.author))
    data.db.conn.commit()

    data.reply('Created new clip {0}.'.format(name))
    data.audio.play(name)


# Youtube query command.

desc = 'Queries a random YouTube link'

def execute(data, argv):
    # choose a random youtube link 

    while True:
        c = data.db.conn.cursor()
        c.execute('SELECT * FROM links ORDER BY random() LIMIT 1')

        row = c.fetchone()

        if row is None:
            data.reply('No links!')
            break
        else:
            if 'youtube.com' in row[1]:
                data.bcast('A gift from <a href="{0}">{1}</a>'.format(row[1], row[0]))
                break
            else:
                continue